{{ '{#start_subject#}' }}{% block subject %}{% endblock %}{{ '{#end_subject#}' }}

{{ '{#start_body#}' }}{% block body %}{% endblock %}{{ '{#end_body#}' }}

{{ '{#start_html#}' }}{% block html %}{% endblock %}{{ '{#end_html#}' }}
