{% extends "mail_templated/base.tpl" %}

{% block subject %}
Hello {{ name }}
{% endblock %}

{% block body %}
{{ name }}, this is a plain text part.
{% endblock %}

{% block html %}
{{ name }}, this is an html part.
{% endblock %}
